==> org.eclipse.papyrus.designer.transformation.languages.cpp.library.source_2.1.0.202201261931.jar: org/eclipse/papyrus/designer/transformation/languages/cpp/library/BootLoaderGenCpp.java <==
/**
 * Copyright (c) 2013 CEA LIST.
 * 
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *  Ansgar Radermacher  ansgar.radermacher@cea.fr
 */
package org.eclipse.papyrus.designer.transformation.languages.cpp.library;

import java.util.List;
import java.util.Stack;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.papyrus.designer.deployment.tools.DepUtils;
import org.eclipse.papyrus.designer.languages.common.base.ElementUtils;
import org.eclipse.papyrus.designer.languages.cpp.profile.C_Cpp.Include;
import org.eclipse.papyrus.designer.transformation.base.utils.LibraryUtils;
import org.eclipse.papyrus.designer.transformation.base.utils.ModelManagement;
import org.eclipse.papyrus.designer.transformation.base.utils.TransformationException;
import org.eclipse.papyrus.designer.transformation.core.Messages;
import org.eclipse.papyrus.designer.transformation.core.m2minterfaces.IM2MTrafoCDP;
import org.eclipse.papyrus.designer.transformation.core.transformations.LazyCopier;
import org.eclipse.papyrus.designer.transformation.core.transformations.TransformationContext;
import org.eclipse.papyrus.designer.transformation.library.transformations.AbstractBootLoaderGen;
import org.eclipse.papyrus.designer.transformation.profile.Transformation.M2MTrafo;
import org.eclipse.papyrus.uml.tools.utils.StereotypeUtil;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.ListExtensions;

/**
 * Create a BootLoader for C++
 */
@SuppressWarnings("all")
public class BootLoaderGenCpp extends AbstractBootLoaderGen implements IM2MTrafoCDP {
  public static final URI CPP_BOOTLOADER_URI = URI.createURI("pathmap://DML_TRAFOS_CPP/BootLoaderCpp.uml");
  
  public static final String CPP_BOOTLOADER_QNAME = "BootLoaderCpp::BootLoader";
  
  public static final String CPP_NODEINFO_QNAME = "BootLoaderCpp::NodeInfo";
  
  /**
   * Create a new boot-loader in a specific package
   * (which represents a node of the system).
   * 
   * @param copier a lazy copier
   * @param nodeIndex the index of the node
   * @param numberOfNodes the number of nodes
   * @throws TransformationException
   */
  @Override
  public void init(final LazyCopier copier, final int nodeIndex, final int numberOfNodes) throws TransformationException {
    this.outputSizeof = false;
    super.init(copier, nodeIndex, numberOfNodes);
    Element _content = LibraryUtils.getContent(BootLoaderGenCpp.CPP_BOOTLOADER_URI, ModelManagement.getResourceSet());
    final org.eclipse.uml2.uml.Package bootloader_ml = ((org.eclipse.uml2.uml.Package) _content);
    if ((bootloader_ml == null)) {
      String _format = String.format(
        Messages.BootLoaderGen_CannotRetrieveTemplate, BootLoaderGenCpp.CPP_BOOTLOADER_URI);
      throw new TransformationException(_format);
    }
    NamedElement _qualifiedElement = ElementUtils.getQualifiedElement(bootloader_ml, BootLoaderGenCpp.CPP_BOOTLOADER_QNAME);
    final org.eclipse.uml2.uml.Class template = ((org.eclipse.uml2.uml.Class) _qualifiedElement);
    if ((template == null)) {
      String _format_1 = String.format(
        Messages.BootLoaderGen_CannotRetrieveTemplate, BootLoaderGenCpp.CPP_BOOTLOADER_QNAME);
      throw new TransformationException(_format_1);
    }
    this.m_bootLoader = copier.<org.eclipse.uml2.uml.Class>getCopy(template);
    final Include cppInclude = StereotypeUtil.<Include>applyApp(this.m_bootLoader, Include.class);
    if ((cppInclude == null)) {
      throw new TransformationException("Cannot apply cppInclude stereotype. Make sure that the C/C++ profile is applied to your model.");
    }
    final String existingBody = cppInclude.getBody();
    String bodyStr = AbstractBootLoaderGen.EMPTYSTR;
    if (this.outputSizeof) {
      String _bodyStr = bodyStr;
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("#include <iostream>");
      _builder.newLine();
      _builder.append("using namespace std");
      _builder.newLine();
      bodyStr = (_bodyStr + _builder);
    }
    cppInclude.setBody((existingBody + bodyStr));
    NamedElement _qualifiedElement_1 = ElementUtils.getQualifiedElement(bootloader_ml, BootLoaderGenCpp.CPP_NODEINFO_QNAME);
    final org.eclipse.uml2.uml.Class nodeInfo = ((org.eclipse.uml2.uml.Class) _qualifiedElement_1);
    final Property nodeIndexElem = nodeInfo.getAttribute("nodeIndex", null);
    final Property numberOfNodesElem = nodeInfo.getAttribute("numberOfNodes", null);
    ValueSpecification _defaultValue = nodeIndexElem.getDefaultValue();
    ((LiteralInteger) _defaultValue).setValue((nodeIndex + 0));
    ValueSpecification _defaultValue_1 = numberOfNodesElem.getDefaultValue();
    ((LiteralInteger) _defaultValue_1).setValue(numberOfNodes);
    copier.<org.eclipse.uml2.uml.Class>getCopy(nodeInfo);
    if (this.outputSizeof) {
    }
  }
  
  @Override
  public String languageCreateInstance(final InstanceSpecification is, final org.eclipse.uml2.uml.Class implementation) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder.toString();
  }
  
  @Override
  public String languageCreateConn(final String varName) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(varName);
    _builder.append(".");
    _builder.append(AbstractBootLoaderGen.CREATE_CONNECTIONS);
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }
  
  @Override
  public String languageRunStart(final String varName, final boolean useOO) {
    StringConcatenation _builder = new StringConcatenation();
    {
      if (useOO) {
        _builder.append(varName);
        _builder.append(".run();");
        _builder.newLineIfNotEmpty();
      } else {
        _builder.append(varName);
        _builder.append(".get_start()->run();");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder.toString();
  }
  
  @Override
  public String languageAssignRef(final String accessName, final String referenceVarName) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(accessName);
    _builder.append(" = &");
    _builder.append(referenceVarName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }
  
  @Override
  public void instanceConfig(final Stack<Slot> slotPath, final InstanceSpecification instance) throws TransformationException {
    final Slot slot = slotPath.peek();
    final StructuralFeature sf = slot.getDefiningFeature();
    if ((sf == null)) {
      String _format = String.format("A slot for instance %s has no defining feature", instance.getName());
      throw new TransformationException(_format);
    }
    String _name = instance.getName();
    String _plus = (_name + ".");
    String _name_1 = sf.getName();
    final String varName = (_plus + _name_1);
    EList<ValueSpecification> _values = slot.getValues();
    for (final ValueSpecification value : _values) {
      String _stringValue = value.stringValue();
      boolean _tripleNotEquals = (_stringValue != null);
      if (_tripleNotEquals) {
        String _m_initCodeCConfig = this.m_initCodeCConfig;
        String _stringValue_1 = value.stringValue();
        String _plus_1 = ((varName + " = ") + _stringValue_1);
        String _plus_2 = (_plus_1 + AbstractBootLoaderGen.EOL);
        this.m_initCodeCConfig = (_m_initCodeCConfig + _plus_2);
      }
    }
  }
  
  @Override
  public void applyTrafo(final M2MTrafo trafo, final org.eclipse.uml2.uml.Package deploymentPlan) throws TransformationException {
    this.init(TransformationContext.current.copier, 0, 0);
    final Stack<Slot> slotPath = new Stack<Slot>();
    EList<InstanceSpecification> _topLevelInstances = DepUtils.getTopLevelInstances(deploymentPlan);
    for (final InstanceSpecification is : _topLevelInstances) {
      this.addInstance(is, slotPath);
    }
    this.addInit("C/C++");
  }
  
  @Override
  public void languageInit() {
  }
  
  @Override
  public String languageActivation(final org.eclipse.uml2.uml.Class[] activationKeys) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("// activation code");
    _builder.newLine();
    {
      for(final org.eclipse.uml2.uml.Class implementation : activationKeys) {
        final EList<String> varNameList = this.m_activation.get(implementation);
        _builder.newLineIfNotEmpty();
        {
          for(final String varName : varNameList) {
            _builder.append(varName);
            _builder.append(".activate();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder.toString();
  }
  
  @Override
  public String languageDeactivation(final org.eclipse.uml2.uml.Class[] activationKeys) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("// deactivation code (reverse order)");
    _builder.newLine();
    {
      List<org.eclipse.uml2.uml.Class> _reverse = ListExtensions.<org.eclipse.uml2.uml.Class>reverse(((List<org.eclipse.uml2.uml.Class>)Conversions.doWrapArray(activationKeys)));
      for(final org.eclipse.uml2.uml.Class implementation : _reverse) {
        final EList<String> varNameList = this.m_activation.get(implementation);
        _builder.newLineIfNotEmpty();
        {
          for(final String varName : varNameList) {
            _builder.append(varName);
            _builder.append(".deactivate();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder.toString();
  }
  
  @Override
  public String languageDefaultExecCode() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("// no user entry point => enter endless loop");
    _builder.newLine();
    _builder.append("for (;;) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("usleep(100);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder.toString();
  }
}

==> org.eclipse.papyrus.designer.transformation.languages.cpp.library.source_3.0.0.202304050728.jar: org/eclipse/papyrus/designer/transformation/languages/cpp/library/BootLoaderGenCpp.java <==
/**
 * Copyright (c) 2013 CEA LIST.
 * 
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *  Ansgar Radermacher  ansgar.radermacher@cea.fr
 */
package org.eclipse.papyrus.designer.transformation.languages.cpp.library;

import java.util.List;
import java.util.Stack;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.papyrus.designer.deployment.tools.DepUtils;
import org.eclipse.papyrus.designer.languages.cpp.profile.C_Cpp.Include;
import org.eclipse.papyrus.designer.transformation.base.utils.LibraryUtils;
import org.eclipse.papyrus.designer.transformation.base.utils.ModelManagement;
import org.eclipse.papyrus.designer.transformation.base.utils.TransformationException;
import org.eclipse.papyrus.designer.transformation.core.Messages;
import org.eclipse.papyrus.designer.transformation.core.m2minterfaces.IM2MTrafoCDP;
import org.eclipse.papyrus.designer.transformation.core.transformations.LazyCopier;
import org.eclipse.papyrus.designer.transformation.core.transformations.TransformationContext;
import org.eclipse.papyrus.designer.transformation.library.transformations.AbstractBootLoaderGen;
import org.eclipse.papyrus.designer.transformation.profile.Transformation.M2MTrafo;
import org.eclipse.papyrus.designer.uml.tools.utils.ElementUtils;
import org.eclipse.papyrus.designer.uml.tools.utils.StereotypeUtil;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.StructuralFeature;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.ListExtensions;

/**
 * Create a BootLoader for C++
 */
@SuppressWarnings("all")
public class BootLoaderGenCpp extends AbstractBootLoaderGen implements IM2MTrafoCDP {
  public static final URI CPP_BOOTLOADER_URI = URI.createURI("pathmap://DML_TRAFOS_CPP/BootLoaderCpp.uml");

  public static final String CPP_BOOTLOADER_QNAME = "BootLoaderCpp::BootLoader";

  public static final String CPP_NODEINFO_QNAME = "BootLoaderCpp::NodeInfo";

  /**
   * Create a new boot-loader in a specific package
   * (which represents a node of the system).
   * 
   * @param copier a lazy copier
   * @param nodeIndex the index of the node
   * @param numberOfNodes the number of nodes
   * @throws TransformationException
   */
  @Override
  public void init(final LazyCopier copier, final int nodeIndex, final int numberOfNodes) throws TransformationException {
    this.outputSizeof = false;
    super.init(copier, nodeIndex, numberOfNodes);
    Element _content = LibraryUtils.getContent(BootLoaderGenCpp.CPP_BOOTLOADER_URI, ModelManagement.getResourceSet());
    final org.eclipse.uml2.uml.Package bootloader_ml = ((org.eclipse.uml2.uml.Package) _content);
    if ((bootloader_ml == null)) {
      String _format = String.format(
        Messages.BootLoaderGen_CannotRetrieveTemplate, BootLoaderGenCpp.CPP_BOOTLOADER_URI);
      throw new TransformationException(_format);
    }
    NamedElement _qualifiedElement = ElementUtils.getQualifiedElement(bootloader_ml, BootLoaderGenCpp.CPP_BOOTLOADER_QNAME);
    final org.eclipse.uml2.uml.Class template = ((org.eclipse.uml2.uml.Class) _qualifiedElement);
    if ((template == null)) {
      String _format_1 = String.format(
        Messages.BootLoaderGen_CannotRetrieveTemplate, BootLoaderGenCpp.CPP_BOOTLOADER_QNAME);
      throw new TransformationException(_format_1);
    }
    this.m_bootLoader = copier.<org.eclipse.uml2.uml.Class>getCopy(template);
    final Include cppInclude = StereotypeUtil.<Include>applyApp(this.m_bootLoader, Include.class);
    if ((cppInclude == null)) {
      throw new TransformationException("Cannot apply cppInclude stereotype. Make sure that the C/C++ profile is applied to your model.");
    }
    final String existingBody = cppInclude.getBody();
    String bodyStr = AbstractBootLoaderGen.EMPTYSTR;
    if (this.outputSizeof) {
      String _bodyStr = bodyStr;
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("#include <iostream>");
      _builder.newLine();
      _builder.append("using namespace std");
      _builder.newLine();
      bodyStr = (_bodyStr + _builder);
    }
    cppInclude.setBody((existingBody + bodyStr));
    NamedElement _qualifiedElement_1 = ElementUtils.getQualifiedElement(bootloader_ml, BootLoaderGenCpp.CPP_NODEINFO_QNAME);
    final org.eclipse.uml2.uml.Class nodeInfo = ((org.eclipse.uml2.uml.Class) _qualifiedElement_1);
    final Property nodeIndexElem = nodeInfo.getAttribute("nodeIndex", null);
    final Property numberOfNodesElem = nodeInfo.getAttribute("numberOfNodes", null);
    ValueSpecification _defaultValue = nodeIndexElem.getDefaultValue();
    ((LiteralInteger) _defaultValue).setValue((nodeIndex + 0));
    ValueSpecification _defaultValue_1 = numberOfNodesElem.getDefaultValue();
    ((LiteralInteger) _defaultValue_1).setValue(numberOfNodes);
    copier.<org.eclipse.uml2.uml.Class>getCopy(nodeInfo);
    if (this.outputSizeof) {
    }
  }

  @Override
  public String languageCreateInstance(final InstanceSpecification is, final org.eclipse.uml2.uml.Class implementation) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder.toString();
  }

  @Override
  public String languageCreateConn(final String varName) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(varName);
    _builder.append(".");
    _builder.append(AbstractBootLoaderGen.CREATE_CONNECTIONS);
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }

  @Override
  public String languageRunStart(final String varName, final boolean useOO) {
    StringConcatenation _builder = new StringConcatenation();
    {
      if (useOO) {
        _builder.append(varName);
        _builder.append(".run();");
        _builder.newLineIfNotEmpty();
      } else {
        _builder.append(varName);
        _builder.append(".get_start()->run();");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder.toString();
  }

  @Override
  public String languageAssignRef(final String accessName, final String referenceVarName) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(accessName);
    _builder.append(" = &");
    _builder.append(referenceVarName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }

  @Override
  public void instanceConfig(final Stack<Slot> slotPath, final InstanceSpecification instance) throws TransformationException {
    final Slot slot = slotPath.peek();
    final StructuralFeature sf = slot.getDefiningFeature();
    if ((sf == null)) {
      String _format = String.format("A slot for instance %s has no defining feature", instance.getName());
      throw new TransformationException(_format);
    }
    String _name = instance.getName();
    String _plus = (_name + ".");
    String _name_1 = sf.getName();
    final String varName = (_plus + _name_1);
    EList<ValueSpecification> _values = slot.getValues();
    for (final ValueSpecification value : _values) {
      String _stringValue = value.stringValue();
      boolean _tripleNotEquals = (_stringValue != null);
      if (_tripleNotEquals) {
        String _m_initCodeCConfig = this.m_initCodeCConfig;
        String _stringValue_1 = value.stringValue();
        String _plus_1 = ((varName + " = ") + _stringValue_1);
        String _plus_2 = (_plus_1 + AbstractBootLoaderGen.EOL);
        this.m_initCodeCConfig = (_m_initCodeCConfig + _plus_2);
      }
    }
  }

  @Override
  public void applyTrafo(final M2MTrafo trafo, final org.eclipse.uml2.uml.Package deploymentPlan) throws TransformationException {
    this.init(TransformationContext.current.copier, 0, 0);
    final Stack<Slot> slotPath = new Stack<Slot>();
    EList<InstanceSpecification> _topLevelInstances = DepUtils.getTopLevelInstances(deploymentPlan);
    for (final InstanceSpecification is : _topLevelInstances) {
      this.addInstance(is, slotPath);
    }
    this.addInit("C/C++");
  }

  @Override
  public void languageInit() {
  }

  @Override
  public String languageActivation(final org.eclipse.uml2.uml.Class[] activationKeys) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("// activation code");
    _builder.newLine();
    {
      for(final org.eclipse.uml2.uml.Class implementation : activationKeys) {
        final EList<String> varNameList = this.m_activation.get(implementation);
        _builder.newLineIfNotEmpty();
        {
          for(final String varName : varNameList) {
            _builder.append(varName);
            _builder.append(".activate();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder.toString();
  }

  @Override
  public String languageDeactivation(final org.eclipse.uml2.uml.Class[] activationKeys) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("// deactivation code (reverse order)");
    _builder.newLine();
    {
      List<org.eclipse.uml2.uml.Class> _reverse = ListExtensions.<org.eclipse.uml2.uml.Class>reverse(((List<org.eclipse.uml2.uml.Class>)Conversions.doWrapArray(activationKeys)));
      for(final org.eclipse.uml2.uml.Class implementation : _reverse) {
        final EList<String> varNameList = this.m_activation.get(implementation);
        _builder.newLineIfNotEmpty();
        {
          for(final String varName : varNameList) {
            _builder.append(varName);
            _builder.append(".deactivate();");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder.toString();
  }

  @Override
  public String languageDefaultExecCode() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("// no user entry point => enter endless loop");
    _builder.newLine();
    _builder.append("for (;;) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("usleep(100);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder.toString();
  }
}
